package protocol

import (
    "time"
)

type PacketType int

const (
    ReqUserCertPkt PacketType = iota
    ReqAllMsgPkt
    ReqMsgPkt
    SubmitMsgPkt
    SendUserCertPkt
    ServerMsgPkt
)

// Define interfaces for packet bodies
type (
    RequestUserCertPacket struct {
        UID string `json:"uid"`
    }

    RequestAllMsgPacket struct {
        FromUID string `json:"from_uid"`
    }

    RequestMsgPacket struct {
        Num uint16 `json:"num"`
    }

    SubmitMessagePacket struct {
        ToUID   string `json:"to_uid"`
        Content []byte `json:"content"`
    }

    SendUserCertPacket struct {
        UID string `json:"uid"`
        Key []byte `json:"key"`
    }

    ServerMessagePacket struct {
        FromUID   string    `json:"from_uid"`
        ToUID     string    `json:"to_uid"`
        Content   []byte    `json:"content"`
        Timestamp time.Time `json:"timestamp"`
    }
)

type PacketBody interface{}

type Packet struct {
    Flag PacketType `json:"flag"`
    Body PacketBody `json:"body"`
}

func NewRequestUserCertPacket(UID string) Packet {
    return Packet{
        Flag: ReqUserCertPkt,
        Body: RequestUserCertPacket{
            UID: UID,
        },
    }
}

func NewRequestAllMsgPacket(fromUID string) Packet {
    return Packet{
        Flag: ReqAllMsgPkt,
        Body: RequestAllMsgPacket{
            FromUID: fromUID,
        },
    }
}

func NewRequestMsgPacket(num uint16) Packet {
    return Packet{
        Flag: ReqMsgPkt,
        Body: RequestMsgPacket{
            Num: num,
        },
    }
}

func NewSubmitMessagePacket(toUID string, content []byte) Packet {
    return Packet{
        Flag: SubmitMsgPkt,
        Body: SubmitMessagePacket{
            ToUID:   toUID,
            Content: content,
        },
    }
}

func NewSendUserCertPacket(uid string, key []byte) Packet {
    return Packet{
        Flag: SendUserCertPkt,
        Body: SendUserCertPacket{
            UID: uid,
            Key: key,
        },
    }
}

func NewServerMessagePacket(fromUID, toUID string, content []byte, timestamp time.Time) Packet {
    return Packet{
        Flag: ServerMsgPkt,
        Body: ServerMessagePacket{
            FromUID:   fromUID,
            ToUID:     toUID,
            Content:   content,
            Timestamp: timestamp,
        },
    }
}
